/*
 *	Micro RTX bindings for the gnu C compiler
 *
 *	  ++jrb		bammi@dsrgsun.ces.cwru.edu
 *
 *  preprocessor symbols:
 *	__NO_INLINE__	if #define'd do not generate inline traps. use
 *			functional form of bindings in grtxbind.c (32 bit)
 *			or srtxtraps.s (16 bit default ints).
 *			you'll need to #define this when using some
 *			debuggers, that otherwise get confused by
 *			inline traps.
 *
 *	__MSHORT__	automatically defined when compiling with 16
 *			bit integer defaults for `int'. automatically
 *			undefined when compiling for 32 bit ints.
 *
 */
#ifndef _RTXBIND_H
#define _RTXBIND_H

#include <basepage.h>
#include <osbind.h>

#if ((defined(__GNUC__)) && (!defined(__NO_INLINE__)))
/*
 * GNU C (pseudo inline) Statement Exprs for traps
 *	note: use of statement expr instead of inline routines
 *	due to brain damage in gcc 1.36.
 *
 *	to see the actual rtxbindings search for the string 'data struct'
 *	forward in this file
 */

#define _trap5_w(n)							\
({									\
	register long retvalue __asm__("d0");				\
	    								\
	__asm__ volatile						\
	("\
		movw    %1,sp@-; \
		trap    #5;	 \
		addqw   #2,sp "						\
	: "=r"(retvalue)			/* outputs */		\
	: "g"(n)				/* inputs  */		\
	: "d0", "d1", "d2", "a0", "a1", "a2"    /* clobbered regs */	\
	);								\
	retvalue;							\
})

#define _trap5_wl(n, l1)						\
({									\
	register long retvalue __asm__("d0");				\
	long  _l1 = (long) (l1);					\
	    								\
	__asm__ volatile						\
	("\
 		movl	%2,sp@-; \
		movw    %1,sp@-; \
		trap    #5;	 \
		addqw   #6,sp "						\
	: "=r"(retvalue)			/* outputs */		\
	: "g"(n), "r"(_l1)			/* inputs  */		\
	: "d0", "d1", "d2", "a0", "a1", "a2"    /* clobbered regs */	\
	);								\
	retvalue;							\
})

#define _trap5_wlw(n, l1, w1)						\
({									\
	register long retvalue __asm__("d0");				\
	long  _l1 = (long) (l1);					\
	short _w1 = (short)(w1);					\
	    								\
	__asm__ volatile						\
	("\
 		movw	%3,sp@-; \
 		movl	%2,sp@-; \
		movw    %1,sp@-; \
		trap    #5;	 \
		addqw   #8,sp "						\
	: "=r"(retvalue)			/* outputs */		\
	: "g"(n), "r"(_l1), "r"(_w1)		/* inputs  */		\
	: "d0", "d1", "d2", "a0", "a1", "a2"    /* clobbered regs */	\
	);								\
	retvalue;							\
})

#define _trap5_wlwl(n, l1, w1, l2)					\
({									\
	register long retvalue __asm__("d0");				\
	long  _l1 = (long) (l1);					\
	short _w1 = (short)(w1);					\
	long  _l2 = (long) (l2);					\
	    								\
	__asm__ volatile						\
	("\
 		movl	%4,sp@-; \
 		movw	%3,sp@-; \
 		movl	%2,sp@-; \
		movw    %1,sp@-; \
		trap    #5;	 \
		lea     sp@(12),sp "					\
	: "=r"(retvalue)			/* outputs */		\
	: "g"(n), "r"(_l1), "r"(_w1), "r"(_l2)	/* inputs  */		\
	: "d0", "d1", "d2", "a0", "a1", "a2"    /* clobbered regs */	\
	);								\
	retvalue;							\
})

#define _trap5_wll(n, l1, l2)						\
({									\
	register long retvalue __asm__("d0");				\
	long  _l1 = (long) (l1);					\
	long  _l2 = (long) (l2);					\
	    								\
	__asm__ volatile						\
	("\
 		movl	%3,sp@-; \
 		movl	%2,sp@-; \
		movw    %1,sp@-; \
		trap    #5;	 \
		lea     sp@(10),sp "					\
	: "=r"(retvalue)			/* outputs */		\
	: "g"(n), "r"(_l1), "r"(_l2)		/* inputs  */		\
	: "d0", "d1", "d2", "a0", "a1", "a2"    /* clobbered regs */	\
	);								\
	retvalue;							\
})

#define _trap5_wlll(n, l1, l2, l3)					\
({									\
	register long retvalue __asm__("d0");				\
	long  _l1 = (long) (l1);					\
	long  _l2 = (long) (l2);					\
	long  _l3 = (long) (l3);					\
	    								\
	__asm__ volatile						\
	("\
 		movl	%4,sp@-; \
 		movl	%3,sp@-; \
 		movl	%2,sp@-; \
		movw    %1,sp@-; \
		trap    #5;	 \
		lea     sp@(14),sp "					\
	: "=r"(retvalue)			/* outputs */		\
	: "g"(n), "r"(_l1), "r"(_l2), "r"(_l3)	/* inputs  */		\
	: "d0", "d1", "d2", "a0", "a1", "a2"    /* clobbered regs */	\
	);								\
	retvalue;							\
})

#define _trap5_wwl(n, w1, l1)						\
({									\
	register long retvalue __asm__("d0");				\
	short _w1 = (short)(w1);					\
	long  _l1 = (long) (l1);					\
	    								\
	__asm__ volatile						\
	("\
 		movl	%3,sp@-; \
 		movw	%2,sp@-; \
		movw    %1,sp@-; \
		trap    #5;	 \
		addqw   #8,sp "						\
	: "=r"(retvalue)			/* outputs */		\
	: "g"(n), "r"(_w1), "r"(_l1)		/* inputs  */		\
	: "d0", "d1", "d2", "a0", "a1", "a2"    /* clobbered regs */	\
	);								\
	retvalue;							\
})

#define _trap5_wllwl(n, l1, l2, w1, l3)					\
({									\
	register long retvalue __asm__("d0");				\
	volatile long  _l1 = (volatile long) (l1);			\
	volatile long  _l2 = (volatile long) (l2);			\
	volatile short _w1 = (volatile short)(w1);			\
	volatile long  _l3 = (volatile long) (l3);			\
	    								\
	__asm__ volatile						\
	("\
 		movl	%4,sp@-; \
 		movw	%3,sp@-; \
		movl    %2,sp@-; \
 		movl	%1,sp@-; \
		movw    %0,sp@- "					\
	: 						 /* outputs */	\
	: "g"(n), "g"(_l1), "g"(_l2), "g"(_w1), "g"(_l3) /* inputs  */	\
	);								\
	__asm__ volatile						\
	("\
		trap    #5;	 \
		lea     sp@(16),sp "					\
	: "=r"(retvalue)				/* outputs */	\
	: 						/* inputs  */	\
	: "d0", "d1", "d2", "a0", "a1", "a2"    /* clobbered regs */    \
	);								\
	retvalue;							\
})

#define _trap5_wlwwlwll( n, l1, w1, w2, l2, w3, l3, l4)			\
({									\
	register long retvalue __asm__("d0");				\
	volatile long  _l1 = (volatile long) (l1);			\
	volatile short _w1 = (volatile short)(w1);			\
	volatile short _w2 = (volatile short)(w2);			\
	volatile long  _l2 = (volatile long) (l2);			\
	volatile short _w3 = (volatile short)(w3);			\
	volatile long  _l3 = (volatile long) (l3);			\
	volatile long  _l4 = (volatile long) (l4);			\
	    								\
	__asm__ volatile						\
	("\
 		movl	%4,sp@-; \
 		movl	%3,sp@-; \
		movw    %2,sp@-; \
 		movl	%1,sp@-; \
		movw    %0,sp@-"					\
	: 						    /* outputs */ \
	: "g"(_w2), "g"(_l2), "g"(_w3), "g"(_l3), "g"(_l4) /* inputs  */ \
	);								\
	__asm__ volatile						\
	("\
 		movw	%3,sp@-; \
 		movl	%2,sp@-; \
		movw    %1,sp@-; \
		trap    #5;	 \
		lea     sp@(24),sp "					\
	: "=r"(retvalue)			/* outputs */		\
	: "g"(n), "g"(_l1), "g"(_w1)		/* inputs  */		\
	: "d0", "d1", "d2", "a0", "a1", "a2"    /* clobbered regs */	\
	);								\
	retvalue;							\
})

#define _trap5_wwllllll(n, w1, l1, l2, l3, l4, l5, l6)			\
({									\
	register long retvalue __asm__("d0");				\
	volatile short _w1 = (volatile short)(w1);			\
	volatile long  _l1 = (volatile long) (l1);			\
	volatile long  _l2 = (volatile long) (l2);			\
	volatile long  _l3 = (volatile long) (l3);			\
	volatile long  _l4 = (volatile long) (l4);			\
	volatile long  _l5 = (volatile long) (l5);			\
	volatile long  _l6 = (volatile long) (l6);			\
	    								\
	__asm__ volatile						\
	("\
 		movl	%4,sp@-; \
 		movl	%3,sp@-; \
		movl    %2,sp@-; \
 		movl	%1,sp@-; \
		movl    %0,sp@-"					\
	: 						    /* outputs */ \
	: "g"(_l2), "g"(_l3), "g"(_l4), "g"(_l5), "g"(_l6) /* inputs  */ \
	);								\
	__asm__ volatile						\
	("\
 		movl	%3,sp@-; \
 		movw	%2,sp@-; \
		movw    %1,sp@-; \
		trap    #5;	 \
		lea     sp@(28),sp "					\
	: "=r"(retvalue)			/* outputs */		\
	: "g"(n), "g"(_w1), "g"(_l1)		/* inputs  */		\
	: "d0", "d1", "d2", "a0", "a1", "a2"    /* clobbered regs */	\
	);								\
	retvalue;							\
})

/*
 * additional gemdos bindings (not in <osbind.h>)
 */
#define trap_1_wwwll(n, w1, w2, l1, l2)					\
({									\
	register long retvalue __asm__("d0");				\
	volatile short _w1 = (volatile short)(w1);			\
	volatile short _w2 = (volatile short)(w2);			\
	volatile long  _l1 = (volatile long) (l1);			\
	volatile long  _l2 = (volatile long) (l2);			\
									\
	__asm__ volatile						\
	("\
		movl    %4,sp@-; \
		movl    %3,sp@-; \
		movw    %2,sp@-; \
		movw    %1,sp@-; \
		movw    %0,sp@- "					\
	:					         /* outputs */	\
	: "g"(n), "g"(_w1), "g"(_w2), "g"(_l1), "g"(_l2) /* inputs  */	\
	);								\
	__asm__ volatile						\
	("\
		trap    #1;	 \
		lea    sp@(14),sp" 					\
	: "=r"(retvalue)			 /* outputs */		\
	: 					/* inputs  */		\
	: "d0", "d1", "d2", "a0", "a1", "a2"    /* clobbered regs */    \
	);								\
	retvalue;							\
})

#else /* __GNUC__ && !__NO_INLINE__ */

/* if inlines are not allowed, then declare things external */

long _trap5_w(short n);							
long _trap5_wl(short n, long l1);
long _trap5_wlw(short n, long l1, short w1);
long _trap5_wlwl(short n, long l1, short w1, long l2);			
long _trap5_wll(short n, long l1, long l2);				
long _trap5_wlll(short n, long l1, long l2, long l3);			
long _trap5_wwl(short n, short w1, long l1);			
long _trap5_wllwl(short n, long l1, long l2, short w1, long l3);		
long _trap5_wlwwlwll(short n, long l1, short w1, short w2, long l2, short w3,
		     long l3, long l4); 
long _trap5_wwllllll(short n, short w1, long l1, long l2, long l3, long l4,
		     long l5, long l6);
long trap_1_wwwll(short n, short w1, short w2, long l1, long l2);	

#endif /* !__NO_INLINE__ */

/*
 * data structs
 */
typedef struct {
    BASEPAGE *basepage;		/* gemdos basepage (extern BASEPAGE *_base) */
    short max_proc;		/* max processes        	            */
    short max_msgs;		/* max system message buffers  	            */
    short max_queues;		/* max message queue        	            */
    void (*create_call)();	/* process creation hook 	*/
    void (*delete_call)();	/* process delete hook		*/
    void (*switch_call)();	/* process (re)schedule hook	*/
} CONFIG;

typedef struct {		/* RTX process status */
    unsigned short	unused1:10;
    unsigned short	EWAIT:1;	/* blocked for event */
    unsigned short	QWAIT:1;	/* blocked for queue */
    unsigned short	PAUSED:1;	/* blocked for timer */
    unsigned short	RUNNING:1;	/* running	     */
    unsigned short	READY:1;	/* ready to run	     */
    unsigned short	unused2:1;
} PROCESS_STATE;

typedef struct {	/* RTX sgtty */
    unsigned short	unused1:7;
    unsigned short	FD_RAWIO:1;
    unsigned short	FD_NOECHO:1;
    unsigned short	FD_CLOSED:1;
    unsigned short	FD_PIPE:1;
    unsigned short	FD_STD:1;
    unsigned short	FD_WRITE:1;
    unsigned short	FD_READ:1;
    unsigned short	FD_FILE:1;
    unsigned short	FD_TTY:1;
} RTXTTY;

typedef struct _pcb 
{
    short	drive;		/* process drive 		*/
    char	path[64];	/* process path  		*/
    char	*dta;		/* process dta			*/
    short	io_flags;	/* process i/o flags		*/
    short	reserved_0;
    long	countdown;	/* p_pause countdown		*/
    BASEPAGE	*basepage;	/* gemdos psp			*/
    struct _pcb *parent;	/* Rtx parent process		*/
    char	*msgbuf;	/* message buffer		*/
    short	sigbits;	/* signal bits			*/
    short	sigmask;	/* signal mask			*/
    short	(*sigfnc[4])(); /* signal handlers		*/
    char	*ssptop;	/* supervisor stack		*/
    char	du_resvd1[2];
    long	tpasize;	/* sizof processes tpa		*/
    long	du_resvd2;
    char	du_resv2[12];
    char	xpath[64];
    char	*memlist;	/* owner memory list		*/
    short	stdtab[6];	/* standard handles		*/
    long	du_resvd3;
    short	*e_mask;	/* event mask			*/
    short	e_flags;	/* event flags			*/
    char	name[14];	/* process name			*/
    char	*vector;	/* user vector			*/
} PCB;


/*
 * Bindings
 */

#define rtx_install(conf) \
    (char *)_trap5_wl((short)0x00, (long)(conf))

#define p_create(name, priority, slice, load, arg_count, args, stack_size)  \
    (char *)_trap5_wlwwlwll((short)0x01, (long)(name), (short)(priority), \
			    (short)(slice), (long)(load),		    \
			    (short)(arg_count), (long)(args),             \
			    (long)(stack_size))
#define 	p_delete(pid) \
    (short)_trap5_wl((short)0x02, (long)(pid))

#define 	p_priority(pid, delta) \
    (short)_trap5_wlw((short)0x03, (long)(pid), (short)(delta))

#define		p_slice(pid, delta) \
    (short)_trap5_wlw((short)0x04, (long)(pid), (short)(delta))

#define		q_create(name, mode) \
    (char *)_trap5_wlw((short)0x05, (long)(name), (short)(mode))

#define 	q_delete(qid) \
    (short)_trap5_wl((short)0x06, (long)(qid))

#define		q_send(qid, msg) \
    (short)_trap5_wll((short)0x07, (long)(qid), (long)(msg))

#define		q_req(qid, msg, nowait_option, timeout) \
    (short)_trap5_wllwl((short)0x08, (long)(qid), (long)(msg), \
                  	(short)(nowait_option), (long)(timeout))

#define		q_jam(qid, msg) \
    (short)_trap5_wll((short)0x09, (long)(qid), (long)(msg))

#define		e_signal(pid, event) \
    (short)_trap5_wlw((short)0x0a, (long)(pid), (short)(event))

#define		e_wait(event, condition, timeout) \
    (short)_trap5_wlwl((short)0x0b, (long)(event), (short)(condition), \
		       (long)(timeout))

#define		p_pause(msec) \
    (short)_trap5_wl((short)0x0c, (long)(msec))

#define		m_alloc(size) \
    (void *)_trap5_wl((short)0x0d, (unsigned long)(size))

#define		m_free(segment) \
    (short)_trap5_wl((short)0x0e, (long)(segment))

#define 	m_assign(segment, pid) \
    (short)_trap5_wll((short)0x0f, (long)(segment), (long)(pid))

#define		 p_lookup(name) \
    (char *)_trap5_wl((short)0x010, (long)(name))

#define		 q_lookup(name) \
    (char *)_trap5_wl((short)0x011, (long)(name))


#define		p_info(pid, state) \
    (char *)_trap5_wll((short)0x012, (long)(pid), (long)(state))

#define 	p_vector(pid, vector) \
    (void *)_trap5_wll((short)0x013, (long)(pid), (long)(vector))

#define		p_suspend(pid)  /* ret type in manual does'nt jive */ \
    (short)_trap5_wl((short)0x014, (long)(pid))

#define		p_resume(pid)   /* ret type in manual does'nt jive */ \
    (short)_trap5_wl((short)0x015, (long)(pid))

#define		d_install(unit, name, constat, conin, conout, costat, cntrl) \
    (short)_trap5_wwllllll((short)0x016, (short)(unit), (long)(name), \
			   (long)(constat), (long)(conin),    \
			   (long)(conout),  (long)(costat),   \
			   (long)(cntrl))

#define		d_cntrl(unit, parm) \
    (short)_trap5_wwl((short)0x017, (short)(unit), (long)(parm))

#define		q_info(qid, proc_q, msg_q) \
    (short)_trap5_wlll((short)0x018, (long)(qid), (long)(proc_q), \
		       (long)(msg_q))

#define 	rtx_remove() \
    (short)_trap5_w((short)0x0ff)

#define		Popen(fd) \
    (short)trap_1_wl((short)0x100,(long)(fd))

#define 	Ftype(handle) \
    (short)trap_1_ww((short)(0x101),(short)(handle))

#define		Flock(fd, flag, offset, length) \
    (short)trap_1_wwwll((short)(0x102), (short)(fd), (short)(flag), \
			(long)(offset), (long)(length))


#define		Fcntrl(fd, mode) \
    (RTXTTY)trap_1_www((short)(0x104), (short)(fd), (RTXTTY)(mode))
    
#define 	Psettpa(size) \
    (long)trap_1_wl((short)(0x103),(long)(size))

#define 	Mquota(a) \
    (long)trap_1_wl((short)(0x105), (long)(a))	/* is this correct?? */

#endif /* _RTXBIND_H */
